@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2007 Quentin Sculo <squentin@free.fr>
+# Copyright (C) 2005-2010 Quentin Sculo <squentin@free.fr>
 #
 # This file is part of Gmusicbrowser.
 # Gmusicbrowser is free software; you can redistribute it and/or modify
@@ -9,24 +9,15 @@ package Play_GST;
 use strict;
 use warnings;
 
-my ($GST_ok,$GST_visuals_ok,$GST_EQ_ok,$GST_RG_ok,$GST_RGA_ok);
+my ($GST_ok,$GST_visuals_ok,$GST_EQ_ok,$GST_RG_ok,$playbin2_ok); our $GST_RGA_ok;
 my ($PlayBin,$Sink);
 my ($WatchTag,$Skip);
-my ($Mute,$Volume);
 my (%Plugins,%Sinks);
 my ($VSink,$visual_window);
+my $AlreadyNextSong;
+my $RG_dialog;
 
-my $RGA_pipeline; my $RG_dialog;
-my $RGA_songmenu=
-{ label => _"Replaygain analysis",	notempty => 'IDs', notmode => 'P', test => sub {$GST_RGA_ok && $::Options{gst_rg_songmenu}; },
- submenu =>
- [	{ label => _"Scan this file",	code => sub { RGA_ReplayGainAnalyse ($_[0]{IDs}); },		onlyone => 'IDs', },
-	{ label => _"Scan per-file track gain",	code => sub { RGA_ReplayGainAnalyse ($_[0]{IDs}); },	onlymany => 'IDs', },
-	{ label => _"Scan using tag-defined album",	code => sub { RGA_ReplayGainAnalyse_byAlbum ($_[0]{IDs}); },	onlymany => 'IDs',  },
-	{ label => _"Scan as an album",	code => sub { RGA_ReplayGainAnalyse([ $_[0]{IDs} ]); },		onlymany=> 'IDs',  },
- ],
-};
-push @::SongCMenu,$RGA_songmenu;
+$::PlayPacks{Play_GST}=1; #register the package
 
 my $reg_keep; #work-around to keep the register from being finalized in gstreamer<0.10.4 (see http://bugzilla.gnome.org/show_bug.cgi?id=324818)
 
@@ -41,12 +32,17 @@ BEGIN
 	sdlaudio	=> {},
 	gconfaudio	=> { name => _"use gnome settings"},
 	halaudio	=> { name => "HAL device", option=>'udi'},
-	pulse		=> { option=>'server device'},
-	jackaudio	=> { name => "jack", option => 'server' },
-#	alsaspdif	=> { option => 'card' },
+	pulse		=> { name => "PulseAudio", option=>'server device'},
+	jackaudio	=> { name => "JACK", option => 'server' },
+	osxaudio	=> {},
+	directsound	=> {},
+	#alsaspdif	=> { name => "alsa S/PDIF", option => 'card' },
+	#oss4		=> {},
+	#nas		=> {},
   );
-  %Plugins=(	mp3	=> 'mad',	ogg => 'vorbisdec',	ape => 'ffdec_ape',
+  %Plugins=(	mp3	=> 'mad',	oga => 'vorbisdec',	ape => 'ffdec_ape',
 		flac	=> 'flacdec',	mpc => 'musepackdec',	wv => 'wavpackdec',
+		m4a	=> 'faad',
 	);
   my $error;
   my $reg;
@@ -57,7 +53,8 @@ BEGIN
 	{	GStreamer->init;
 		$reg=GStreamer::Registry->get_default;
 		$reg_keep=$reg if GStreamer->CHECK_VERSION(0,10,4);
-		if ( $reg->lookup_feature('playbin') ) { $GST_ok=1; }
+		$playbin2_ok= $reg->lookup_feature('playbin2');
+		if ( $playbin2_ok || $reg->lookup_feature('playbin') ) { $GST_ok=1; }
 		else { $error="gstreamer plugin 'playbin' not found\nYou need to install at least gst-plugins-base\n"; }
 	}
 	else { $error=$@? $@ : "Can't initialize GStreamer.\n"; }
@@ -66,7 +63,7 @@ BEGIN
   if ($error) {warn "$error -> gstreamer output won't be available.\n"}
   if ($GST_ok)
   {	$::Options{gst_sink}||= (grep ($reg->lookup_feature($_.'sink'), qw/autoaudio gconfaudio alsa esd pulse oss/),'alsa')[0]; #find a default sink
-	if (my $feat=$reg->lookup_feature('equalizer-10bands')) { $GST_EQ_ok=1; }
+	if ($reg->lookup_feature('equalizer-10bands')) { $GST_EQ_ok=1; }
 	else {warn "gstreamer plugin 'equalizer-10bands' not found -> equalizer not available\n";}
 	if ($reg->lookup_feature('rglimiter') && $reg->lookup_feature('rgvolume')) { $GST_RG_ok=1; }
 	else {warn "gstreamer plugins 'rglimiter' and/or 'rgvolume' not found -> replaygain not available\n";}
@@ -97,20 +94,23 @@ sub supported_sinks
 
 sub init
 {	return undef unless $GST_ok;
-	$Volume= $::Options{gst_volume};
-	createPlayBin();
+	#createPlayBin();
 	return bless { EQ=>$GST_EQ_ok, visuals => $GST_visuals_ok },__PACKAGE__;
 }
 
 sub createPlayBin
 {	if ($PlayBin) { $PlayBin->get_bus->remove_signal_watch; }
-	$PlayBin=GStreamer::ElementFactory->make(playbin => 'playbin');
+	my $pb= $playbin2_ok ? 'playbin2' : 'playbin';
+	$PlayBin=GStreamer::ElementFactory->make($pb => 'playbin'); #FIXME only the first one used works
+	$PlayBin->set('flags' => [qw/audio soft-volume/]) if $playbin2_ok;
+	$PlayBin->set(volume => ($::Volume/100)**3); 	#initialize volume, use a cubic volume scale
 	my $bus=$PlayBin->get_bus;
 	$bus->add_signal_watch;
 #	$bus->signal_connect('message' => \&bus_message);
 	$bus->signal_connect('message::eos' => \&bus_message_end);
 	$bus->signal_connect('message::error' => \&bus_message_end,1);
 	$bus->signal_connect('message::state-changed' => \&bus_message_state_changed);
+	$PlayBin->signal_connect(about_to_finish => \&about_to_finish) if $::Options{gst_gapless};
 	if ($visual_window) { create_visuals() }
 }
 
@@ -144,21 +144,22 @@ sub Close
 {	$Sink=undef;
 }
 
-sub GetVolume	{$Volume}
-sub GetMute	{$Mute}
+sub GetVolume	{$::Volume}
+sub GetMute	{$::Mute}
 sub SetVolume
 {	shift;
 	my $set=shift;
-	if	($set eq 'mute')	{ $Mute=$Volume; $Volume=0;	}
-	elsif	($set eq 'unmute')	{ $Volume=$Mute; $Mute=0;	}
-	elsif	($set=~m/^\+(\d+)$/)	{ $Volume+=$1; }
-	elsif	($set=~m/^-(\d+)$/)	{ $Volume-=$1; }
-	elsif	($set=~m/^(\d+)$/)	{ $Volume =$1; }
-	$Volume=0   if $Volume<0;
-	$Volume=100 if $Volume>100;
-	$PlayBin->set(volume => $Volume/100); #or /10 ?
+	if	($set eq 'mute')	{ $::Mute=$::Volume; $::Volume=0;}
+	elsif	($set eq 'unmute')	{ $::Volume=$::Mute; $::Mute=0;  }
+	elsif	($set=~m/^\+(\d+)$/)	{ $::Volume+=$1; }
+	elsif	($set=~m/^-(\d+)$/)	{ $::Volume-=$1; }
+	elsif	($set=~m/(\d+)/)	{ $::Volume =$1; }
+	$::Volume=0   if $::Volume<0;
+	$::Volume=100 if $::Volume>100;
+	$PlayBin->set(volume => ($::Volume/100)**3) if $PlayBin; 	#use a cubic volume scale
 	::HasChanged('Vol');
-	$::Options{gst_volume}=$Volume;
+	$::Options{Volume}=$::Volume;
+	$::Options{Volume_mute}=$::Mute;
 }
 
 sub SkipTo
@@ -195,8 +196,17 @@ sub make_sink
 	return $sink;
 }
 
+sub about_to_finish	#GAPLESS
+{	#warn "-------about_to_finish  $::NextFileToPlay\n";
+	return unless $::NextFileToPlay;
+	set_file($::NextFileToPlay);
+	$AlreadyNextSong=$::NextFileToPlay;
+	$::NextFileToPlay=0;
+}
+
 sub Play
 {	(my($package,$file),$Skip)=@_;
+	#warn "------play $file\n";
 	#$PlayBin->set_state('ready');#&Stop;
 	#my ($ext)=$file=~m/\.([^.]*)$/; warn $ext;
 	#::ErrorPlay('not supported') and return undef  unless $Plugins{$ext};
@@ -206,7 +216,14 @@ sub Play
 	$keep=0 if $Sink->{EQ} xor $useEQ;
 	$keep=0 if $Sink->{RG} xor $useRG;
 	$keep=0 if $package->{modif}; #advanced options changed
-	unless ($keep)
+	if ($AlreadyNextSong && $AlreadyNextSong eq $file && $keep && !$Skip)
+	{	$AlreadyNextSong=undef;
+		return;
+	}
+	if ($keep)
+	{	$package->Stop(1);
+	}
+	else
 	{	createPlayBin();
 		warn "Creating new gstreamer sink\n" if $::debug;
 		delete $package->{modif};
@@ -239,20 +256,22 @@ sub Play
 		}
 		else {$PlayBin->set('audio-sink' => $Sink);}
 	}
-	$PlayBin->set(volume => $Volume/100); #or /10 ?
 
 	if ($visual_window)
 	{	$VSink->set_xwindow_id($visual_window->window->XID);
 	}
 	warn "playing $file\n";
-	my $f=$file;
+	set_file($file);
+	$PlayBin -> set_state('playing');
+	$WatchTag=Glib::Timeout->add(500,\&_UpdateTime) unless $WatchTag;
+}
+sub set_file
+{	my $f=shift;
 	if ($f!~m#^([a-z]+)://#)
 	{	$f=~s#([^A-Za-z0-9- /\.])#sprintf('%%%02X', ord($1))#seg;
 		$f='file://'.$f;
 	}
 	$PlayBin -> set(uri => $f);
-	$PlayBin -> set_state('playing');
-	$WatchTag=Glib::Timeout->add(500,\&_UpdateTime) unless $WatchTag;
 }
 
 sub set_equalizer
@@ -266,7 +285,8 @@ sub set_equalizer
 	::setlocale(::LC_NUMERIC, '');
 }
 sub EQ_Get_Range
-{	my ($min,$max)=(-1,1);
+{	createPlayBin() unless $PlayBin;
+	my ($min,$max)=(-1,1);
 	{	my $equalizer=$PlayBin->get_by_name('equalizer')
 		 || GStreamer::ElementFactory->make('equalizer-10bands' => 'equalizer');
 		last unless $equalizer;
@@ -280,6 +300,7 @@ sub EQ_Get_Range
 }
 sub EQ_Get_Hz
 {	my $i=$_[1];
+	createPlayBin() unless $PlayBin;
 	my $equalizer=$PlayBin->get_by_name('equalizer')
 	 || GStreamer::ElementFactory->make('equalizer-10bands' => 'equalizer');
 	return undef unless $equalizer;
@@ -299,6 +320,7 @@ sub create_visuals
 		$VSink->set_xwindow_id($visual_window->window->XID);
 	}
 	$PlayBin->set('video-sink' => $VSink) if $PlayBin;
+	$PlayBin->set('flags' => [qw/audio vis soft-volume/]) if $PlayBin && $playbin2_ok;
 	set_visual();
 }
 sub add_visuals
@@ -314,6 +336,7 @@ sub add_visuals
 }
 sub remove_visuals
 {	$VSink->set_xwindow_id(0) if $VSink;
+	$PlayBin->set('flags' => [qw/audio soft-volume/]) if $PlayBin && $playbin2_ok;
 	$PlayBin->set('video-sink' => undef) if $PlayBin;
 	$PlayBin->set('vis-plugin' => undef) if $PlayBin;
 	$visual_window=$VSink=undef;
@@ -368,11 +391,12 @@ sub list_visuals
 
 sub _UpdateTime
 {	my ($result,$state,$pending)=$PlayBin->get_state(0);
+	if ($AlreadyNextSong) { warn "UpdateTime: gapless change to next song\n" if $::debug; ::end_of_file_faketime(); return 1; }
 	warn "state: $result,$state,$pending\n" if $::debug;
 	return 1 if $result eq 'async';
 	if ($state ne 'playing' && $state ne 'paused')
 	{	return 1 if $pending eq 'playing' || $pending eq 'paused';
-		::ResetTime() unless $::Play_package ne 'Play_GST';
+		::ResetTime() unless 'Play_GST' ne ref $::Play_package;
 		$WatchTag=undef;
 		return 0;
 	}
@@ -390,7 +414,8 @@ sub Stop
 	#my ($result,$state,$pending)=$PlayBin->get_state(0);
 	#warn "stop: state: $result,$state,$pending\n";
 	#return if $state eq 'null' and $pending eq 'void-pending';
-	$PlayBin->set_state('null');
+	$PlayBin->set_state('null') if $PlayBin;
+	#warn "--stop\n";
 }
 
 sub RG_set_options
@@ -404,27 +429,103 @@ sub RG_set_options
 	$rgv->set('fallback-gain' => $::Options{gst_rg_fallback}||0);
 	#$rgv->set(headroom => $::Options{gst_rg_headroom}||0);
 }
-sub RGA_ReplayGainAnalyse_byAlbum
+
+sub RG_PrefBox
+{	my ($sg1,$sg2)=@_;
+	my $check=::NewPrefCheckButton(gst_use_replaygain => _"Use ReplayGain", tip=>_"Normalize volume (the files must have replaygain tags)");
+	$sg1->add_widget($check);
+	#my $start=Gtk2::Button->new(_ "Start ReplayGain analysis");
+	my $opt  =Gtk2::Button->new(_"ReplayGain options");
+	$sg2->add_widget($opt);
+	$opt->signal_connect(clicked => sub
+		{	if ($RG_dialog) {$RG_dialog->present;return}
+			$RG_dialog= Gtk2::Dialog->new (_"ReplayGain options", undef, [], 'gtk-close' => 'close');
+			$RG_dialog->signal_connect(destroy => sub {$RG_dialog=undef});
+			$RG_dialog->signal_connect(response =>sub {$_[0]->destroy});
+			my $update=sub { RG_set_options(); };
+			my $songmenu=::NewPrefCheckButton(gst_rg_songmenu => _"Show replaygain submenu");
+			my $albummode=::NewPrefCheckButton(gst_rg_albummode => _"Album mode", cb=>$update, tip=>_"Use album normalization instead of track normalization");
+			my $nolimiter=::NewPrefCheckButton(gst_rg_limiter => _"Hard limiter", cb=>$update, tip=>_"Used for clipping prevention");
+			my $sg1=Gtk2::SizeGroup->new('horizontal');
+			my $sg2=Gtk2::SizeGroup->new('horizontal');
+			my $preamp=	::NewPrefSpinButton('gst_rg_preamp',   -60,60, cb=>$update, digits=>1, rate=>.1, step=>.1, text2=> 'dB', sizeg1=>$sg1, sizeg2=>$sg2, text1=>_"pre-amp", tip=>_"Extra gain");
+			my $fallback=	::NewPrefSpinButton('gst_rg_fallback', -60,60, cb=>$update, digits=>1, rate=>.1, step=>.1, text2=> 'dB', sizeg1=>$sg1, sizeg2=>$sg2, text1=>_"fallback-gain", tip=>_"Gain for songs missing replaygain tags");
+			$RG_dialog->vbox->pack_start($_,0,0,2) for $albummode,$preamp,$fallback,$nolimiter,$songmenu;
+			$RG_dialog->show_all;
+
+		});
+	#$start->signal_connect(clicked => sub
+	#	{	my @list; my %done;
+	#		for my $ID (@::Library)
+	#		{	my $albumid=Songs::Get_gid($ID,'album');
+	#			my $l=AA::GetIDs('album',$albumid);
+	#			if (@$l>1 && Songs::Gid_to_get('album',$album) ne '')
+	#			{ push @list,[@$l] unless exists $done{$album}; $done{$aid}=undef; }
+	#			else { push @list,$ID; }
+	#		}
+	#		Analyse(\@list);
+	#	});
+	#$start->signal_connect(clicked => sub { Analyse(\@::Library) });
+	#$start->set_sensitive(0) unless $GST_RGA_ok;
+	my $box=::Hpack($check,$opt);
+	$box->set_sensitive(0) unless $GST_RG_ok;
+	return $box;
+}
+
+sub AdvancedOptions
+{	my $self=$_[0];
+	my $vbox=Gtk2::VBox->new(::FALSE, 2);
+	my $gapless= ::NewPrefCheckButton(gst_gapless => _"enable gapless (experimental)", cb=>sub { $self->{modif}=1 });
+	$gapless->set_sensitive(0) unless $playbin2_ok;
+	$vbox->pack_start($gapless,::FALSE,::FALSE,2);
+	my $sg1=Gtk2::SizeGroup->new('horizontal');
+	my $sg2=Gtk2::SizeGroup->new('horizontal');
+	for my $s (sort grep $Sinks{$_}{ok} && $Sinks{$_}{option}, keys %Sinks)
+	{	my $label= $Sinks{$s}{name}||$s;
+		for my $opt (sort split / /,$Sinks{$s}{option})
+		{	my $hbox=::NewPrefEntry("gst_${s}_$opt", "$s $opt : ", cb => sub { $self->{modif}=1 }, sizeg1 => $sg1, sizeg2 => $sg2);
+			$vbox->pack_start($hbox,::FALSE,::FALSE,2);
+		}
+	}
+	return $vbox;
+}
+
+package GMB::GST_ReplayGain;
+
+my $RGA_pipeline;
+my (@towrite,$writing);
+my $RGA_songmenu=
+{ label => _"Replaygain analysis",	notempty => 'IDs', notmode => 'P', test => sub {$Play_GST::GST_RGA_ok && $::Options{gst_rg_songmenu}; },
+ submenu =>
+ [	{ label => _"Scan this file",			code => sub { Analyse ($_[0]{IDs}); },		onlyone => 'IDs', },
+	{ label => _"Scan per-file track gain",		code => sub { Analyse ($_[0]{IDs}); },		onlymany=> 'IDs', },
+	{ label => _"Scan using tag-defined album", 	code => sub { Analyse_byAlbum ($_[0]{IDs}); },	onlymany=> 'IDs', },
+	{ label => _"Scan as an album",			code => sub { Analyse([ $_[0]{IDs} ]); },	onlymany=> 'IDs', },
+ ],
+};
+push @::SongCMenu,$RGA_songmenu;
+
+sub Analyse_byAlbum
 {	my $IDs=$_[0];
-	::SortList($IDs,::SONG_ALBUM);
-	my @list; my @album; my $album0;
+	Songs::SortList($IDs,'album');
+	my @list; my @album; my $aid0;
 	for my $ID (@$IDs)
-	{	my $album= $::Songs[$ID][::SONG_ALBUM];
-		if (defined $album0)
-		{	if ($album0 eq $album) {push @album,$ID;next}
-			else
-			{	push @list, (@album>1 ? [@album] : $album[0]);
-				$album0=undef; @album=undef;
+	{	my $aid= Songs::Get($ID,'album');
+		if (defined $aid0)
+		{	if ($aid0 eq $aid) {push @album,$ID;next}	# same album
+			else						# different album
+			{	push @list, (@album>1 ? [@album] : $album[0]);	#add songs from previous album as a group
+				$aid0=undef; @album=undef;
 			}
 		}
-		if ($album=~m/^<Unknown>/) {push @list,$ID}
-		else {$album0=$album; @album=($ID)}
+		if ($aid eq '') {push @list,$ID}	# songs with no album -> add song on its own
+		else {$aid0=$aid; @album=($ID)}		# new album
 	}
-	if (defined $album0) { push @list, (@album>1 ? [@album] : $album[0]); }
-	RGA_ReplayGainAnalyse(\@list);
+	if (defined $aid0) { push @list, (@album>1 ? [@album] : $album[0]); } # add songs from last album as a group
+	Analyse(\@list);
 }
-sub RGA_ReplayGainAnalyse
-{	my $IDs=$_[0];
+sub Analyse
+{	my @IDs= ::uniq(@{ $_[0] });
 	unless ($RGA_pipeline)
 	{	$RGA_pipeline=GStreamer::Pipeline->new('RGA_pipeline');
 		my $audiobin=GStreamer::Bin->new('RGA_audiobin');
@@ -438,7 +539,7 @@ sub RGA_ReplayGainAnalyse
 		$audiobin->add_pad(GStreamer::GhostPad ->new('sink', $audiopad));
 		$RGA_pipeline->add($src,$decodebin,$audiobin);
 		$src->link($decodebin);
-		$decodebin->signal_connect(new_decoded_pad => \&RGA_newpad_cb);
+		$decodebin->signal_connect(new_decoded_pad => \&newpad_cb);
 
 		#@elems= map GStreamer::ElementFactory->make($_ => $_), @elems;
 		#$RGA_pipeline->add(@elems);
@@ -447,18 +548,17 @@ sub RGA_ReplayGainAnalyse
 		my $bus=$RGA_pipeline->get_bus;
 		$bus->add_signal_watch;
 		$bus->signal_connect('message::error' => sub { warn "ReplayGain analysis error : ".$_[1]->error."\n"; }); #FIXME
-		$bus->signal_connect('message::tag' => \&RGA_bus_message_tag);
-		$bus->signal_connect('message::eos' => \&RGA_process_next);
+		$bus->signal_connect('message::tag' => \&bus_message_tag);
+		$bus->signal_connect('message::eos' => \&process_next);
 		#FIXME check errors
 	}
-	#FIXME remove duplicates in @$IDs;
-	push @{$RGA_pipeline->{queue}},@$IDs;
-	my $nb=@$IDs; #warn "@$IDs";
-	$nb+=@$_-1 for grep ref, @$IDs; #count tracks in album lists
-	$RGA_pipeline->{total}+=$nb; #warn "$RGA_pipeline->{total}==$nb;\n";
-	RGA_process_next() if $RGA_pipeline->{total}==$nb;
+	push @{$RGA_pipeline->{queue}},@IDs;
+	my $nb=@IDs; #warn "@IDs";
+	$nb+=@$_-1 for grep ref, @IDs; #count tracks in album lists
+	::Progress('replaygain', add=>$nb, abortcb=>\&StopAnalysis, title=>_"Replaygain analysis");
+	process_next() unless $::Progress{replaygain}{current}; #FIXME maybe check if $RGA_pipeline is running instead
 }
-sub RGA_newpad_cb
+sub newpad_cb
 {	my ($decodebin,$pad)=@_;
 	my $audiopad = $RGA_pipeline->get_by_name('RGA_audiobin')->get_pad('sink');
 	return if $audiopad->is_linked;
@@ -467,9 +567,10 @@ sub RGA_newpad_cb
 	return unless $str=~m/audio/;
 	$pad->link($audiopad);
 }
-sub RGA_process_next
-{	return unless $RGA_pipeline;
-	$RGA_pipeline->{done}++;
+sub process_next
+{	::Progress('replaygain', inc=>1) if $_[0]; #called from callback => one file has been scanned => increment
+	unless ($RGA_pipeline) { ::Progress('replaygain', abort=>1); return; }
+
 	my $rganalysis=$RGA_pipeline->get_by_name('rganalysis');
 	my $ID;
 	if (my $list=$RGA_pipeline->{albumIDs})
@@ -488,10 +589,11 @@ sub RGA_process_next
 			$ID=$list->[0];
 			$RGA_pipeline->{album_i}=0;
 		}
-		my $f= $::Songs[$ID][::SONG_PATH].::SLASH.$::Songs[$ID][::SONG_FILE];
+		my $f= Songs::Get($ID,'fullfilename_raw');
+		::_utf8_on($f); # pretend it's utf8 to prevent a conversion to utf8 by the bindings
 		$RGA_pipeline->{ID}=$ID;
 		warn "analysing [$ID] $f\n";
-		$RGA_pipeline->get_by_name('filesrc')->set(location => $f);#set(uri => $f);
+		$RGA_pipeline->get_by_name('filesrc')->set(location => $f);
 		$rganalysis->set_locked_state(0);
 		$RGA_pipeline->set_state('playing');
 	}
@@ -499,16 +601,16 @@ sub RGA_process_next
 	{	$RGA_pipeline->set_state('null');
 		$RGA_pipeline=undef;
 	}
-	::HasChanged('GST_RGAnalysis');
 	1;
 }
-sub RGA_bus_message_tag
+sub bus_message_tag
 {	my $msg=$_[1];
 	my $tags=$msg->tag_list;
 	#for my $key (sort keys %$tags) {warn "key=$key => $tags->{$key}\n"}
 	#FIXME should check if the message comes from the rganalysis element, but not supported by the bindings yet, instead check if any non replaygain tags => will re-write replaygain tags _before_ analysis for files without other tags
+	#return if GStreamer->VERSION >=.10 && $msg->src == $RGA_pipeline->get_by_name('rganalysis'); FIXME with Gstreamer 0.10 : $message->src should work, not tested yet !!!! TESTME
 	return unless exists $tags->{'replaygain-track-gain'};
-	return if grep !m/^replaygain-/, keys %$tags;
+	return if grep !m/^replaygain-/, keys %$tags; #if other tags than replaygain => doesn't come from the rganalysis element
 
 	my $cID=$RGA_pipeline->{ID};
 	if ($::debug)
@@ -520,110 +622,52 @@ sub RGA_bus_message_tag
 	{	$RGA_pipeline->{album_tosave}{ $cID }= [@$tags{'replaygain-track-gain','replaygain-track-peak'}];
 		if (exists $tags->{'replaygain-album-gain'} && !$RGA_pipeline->get_by_name('rganalysis')->get('num-tracks'))
 		{	#album done
-			for my $ID (@{$RGA_pipeline->{albumIDs}})
-			{	@$tags{'replaygain-track-gain','replaygain-track-peak'}= @{$RGA_pipeline->{album_tosave}{$ID}};
-				::setlocale(::LC_NUMERIC, 'C');
-				my @modif= map [$_,0,"$tags->{$_}[0]"], qw/replaygain-reference-level replaygain-track-gain replaygain-track-peak replaygain-album-gain replaygain-album-peak/;
-				::setlocale(::LC_NUMERIC, '');
-				SimpleTagWriting::set($ID, \@modif, \&RGA_write_error);
-				last unless $RGA_pipeline; #in case RGA has been aborted after an error writing a tag
+			my $IDs= $RGA_pipeline->{albumIDs};
+			my $gainpeak= $RGA_pipeline->{album_tosave};
+			for my $ID (@$IDs)
+			{	@$tags{'replaygain-track-gain','replaygain-track-peak'}= @{$gainpeak->{$ID}};
+				queuewrite($ID,$tags,1);
 			}
 		}
 	}
 	else
-	{	::setlocale(::LC_NUMERIC, 'C');
-		my @modif= map [$_,0,"$tags->{$_}[0]"], qw/replaygain-reference-level replaygain-track-gain replaygain-track-peak/;
-		::setlocale(::LC_NUMERIC, '');
-		SimpleTagWriting::set( $cID, \@modif, \&RGA_write_error);
+	{	queuewrite($cID,$tags,0);
 	}
 	1;
 }
-sub RGA_write_error
-{	my $err=_"Error writing replaygain tags :\n".shift;
-	my $abort=_"Abort ReplayGain analysis";
-	my $ret=::Retry_Dialog($err,undef,$abort);
-	if ($ret eq 'abort') {RGA_stop()}
-	return $ret;
-}
-sub RGA_stop
-{	$RGA_pipeline->set_state('null');
-	$RGA_pipeline=undef;
-	::HasChanged('GST_RGAnalysis');
-}
-sub RGA_PrefBox
-{	my $sg1=shift;
-	my $check=::NewPrefCheckButton(gst_use_replaygain => _"Use ReplayGain",undef,_"Normalize volume (the files must have replaygain tags)");
-	$sg1->add_widget($check);
-	#my $start=Gtk2::Button->new(_ "Start ReplayGain analysis");
-	my $stop =Gtk2::Button->new(_"Abort ReplayGain analysis");
-	my $opt  =Gtk2::Button->new(_"ReplayGain options");
-	my $progress=Gtk2::ProgressBar->new;
-	$opt->signal_connect(clicked => sub
-		{	if ($RG_dialog) {$RG_dialog->present;return}
-			$RG_dialog= Gtk2::Dialog->new (_"ReplayGain options", undef, [],
-				'gtk-close' => 'close');
-			$RG_dialog->signal_connect(destroy => sub {$RG_dialog=undef});
-			$RG_dialog->signal_connect(response =>sub {$_[0]->destroy;$RG_dialog=undef});
-			my $update=sub { RG_set_options(); };
-			my $songmenu=::NewPrefCheckButton(gst_rg_songmenu => _"Show replaygain submenu");
-			my $albummode=::NewPrefCheckButton(gst_rg_albummode => _"Album mode",$update,_"Use album normalization instead of track normalization");
-			my $nolimiter=::NewPrefCheckButton(gst_rg_limiter => _"Hard limiter",$update,_"Used for clipping prevention");
-			my $sg1=Gtk2::SizeGroup->new('horizontal');
-			my $sg2=Gtk2::SizeGroup->new('horizontal');
-			my $preamp=::NewPrefSpinButton(gst_rg_preamp => $update, .1,1,-60,60,.1,1,_"pre-amp",'dB',$sg1,$sg2,_"Extra gain");
-			my $fallback=::NewPrefSpinButton(gst_rg_fallback => $update, .1,1,-60,60,.1,1,_"fallback-gain",'dB',$sg1,$sg2,_"Gain for songs missing replaygain tags");
-			$RG_dialog->vbox->pack_start($_,0,0,2) for $albummode,$preamp,$fallback,$nolimiter,$songmenu;
-			$RG_dialog->show_all;
-
-		});
-	#$start->signal_connect(clicked => sub
-	#	{	my @list; my %done;
-	#		for my $ID (@::Library)
-	#		{	my $album=$::Songs[$ID][::SONG_ALBUM];
-	#			my $l=$::Album{$album}[::AALIST];
-	#			if (@$l>1 && $album!~m/^<Unknown>/)
-	#			{ push @list,[@$l] unless exists $done{$album}; $done{$album}=undef; }
-	#			else { push @list,$ID; }
-	#		}
-	#		RGA_ReplayGainAnalyse(\@list);
-	#	});
-	#$start->signal_connect(clicked => sub { RGA_ReplayGainAnalyse(\@::Library) });
-	$stop->signal_connect( clicked => \&RGA_stop);
-	#$start->set_sensitive(0) unless $GST_RGA_ok;
-	my $box=::Hpack($check,$opt,'_',$progress,$stop);
-	$_->set_no_show_all(1) for $stop,$progress;
-	my $update=sub
-		{	my $progress=$_[0];
-			if ($RGA_pipeline)
-			{	$stop->show; $progress->show;
-				my $max=$RGA_pipeline->{total};
-				my $done=$RGA_pipeline->{done};
-				$progress->set_fraction( $done/$max );
-				$progress->set_text( "$done / $max" );
-			}
-			else { $stop->hide; $progress->hide; }
-		};
-	::Watch($progress,'GST_RGAnalysis',$update);
-	&$update($progress);
-	$box->set_sensitive(0) unless $GST_RG_ok;
-	return $box;
-}
-
-sub AdvancedOptions
-{	my $self=$_[0];
-	my $vbox=Gtk2::VBox->new(::FALSE, 2);
-	my $sg1=Gtk2::SizeGroup->new('horizontal');
-	my $sg2=Gtk2::SizeGroup->new('horizontal');
-	for my $s (sort grep $Sinks{$_}{ok} && $Sinks{$_}{option}, keys %Sinks)
-	{	my $label= $Sinks{$s}{name}||$s;
-		for my $opt (sort split / /,$Sinks{$s}{option})
-		{	my $hbox=::NewPrefEntry("gst_$s".'_'.$opt, "$s $opt : ", sub { $self->{modif}=1 },$sg1,$sg2);
-			$vbox->pack_start($hbox,::FALSE,::FALSE,2);
-		}
+sub queuewrite
+{	my ($ID,$tags,$albumtag)=@_;
+	my @keys=qw/replaygain-reference-level replaygain-track-gain replaygain-track-peak/;
+	push @keys, qw/replaygain-album-gain replaygain-album-peak/ if $albumtag;
+	::setlocale(::LC_NUMERIC, 'C');
+	my @modif;
+	for my $key (@keys)
+	{	my $field=$key;
+		$field=~tr/-/_/; # convert replaygain-track-gain to replaygain_track_gain ...
+		push @modif, $field, "$tags->{$key}[0]";#string-ify them with C locale to make sure it's correct
 	}
-	return $vbox;
+	::setlocale(::LC_NUMERIC, '');
+	push @towrite, $ID,\@modif;
+	WriteRGtags() unless $writing;
+}
+sub WriteRGtags
+{	return $writing=0 if !@towrite;
+	$writing=1;
+	my $ID=   shift @towrite;
+	my $modif=shift @towrite;
+	Songs::Set($ID, $modif,
+		abortmsg	=> _"Abort ReplayGain analysis",
+		error_prefix	=> _"Error writing replaygain tags :\n",
+		abortcb		=> \&StopAnalysis,
+		callback_finish	=> \&WriteRGtags,
+	);
+}
+sub StopAnalysis
+{	$RGA_pipeline->set_state('null') if $RGA_pipeline;
+	$RGA_pipeline=undef;
+	::Progress('replaygain', abort=>1);
+	@towrite=();
 }
-
 
 package Play_GST_server;
 use Socket;
@@ -633,6 +677,8 @@ our @ISA=('Play_GST');
 our %sockets;
 my ($stream,$Server);
 
+$::PlayPacks{Play_GST_server}=1; #register the package
+
 sub init
 {	return undef unless $GST_ok;
 	my $ok=1;
