@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2008 Quentin Sculo <squentin@free.fr>
+# Copyright (C) 2005-2009 Quentin Sculo <squentin@free.fr>
 #
 # This file is part of Gmusicbrowser.
 # Gmusicbrowser is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@ package Tag::OGG;
 use strict;
 use warnings;
 use Encode qw(decode encode);
-use bytes;
+use MIME::Base64;
 
 use constant
 { PACKET_INFO	 => 1,
@@ -22,8 +22,16 @@ use constant
 };
 
 my @crc_lookup;
+my $digestcrc;
 INIT
-{ @crc_lookup=
+{ eval
+  {	require Digest::CRC;
+	$digestcrc=Digest::CRC->new(width=>32, init=>0, xorout=>0, poly=>0x04C11DB7, refin=>0, refout=>0);
+	warn "oggheader.pm : using Digest::CRC\n" if $::debug;
+  };
+  if ($@)
+  { warn "oggheader.pm : Digest::CRC not found, using slow pure-perl replacement.\n" if $::debug;
+    @crc_lookup=
  (0x00000000,0x04c11db7,0x09823b6e,0x0d4326d9,
   0x130476dc,0x17c56b6b,0x1a864db2,0x1e475005,
   0x2608edb8,0x22c9f00f,0x2f8ad6d6,0x2b4bcb61,
@@ -88,7 +96,7 @@ INIT
   0x9abc8bd5,0x9e7d9662,0x933eb0bb,0x97ffad0c,
   0xafb010b1,0xab710d06,0xa6322bdf,0xa2f33668,
   0xbcb4666d,0xb8757bda,0xb5365d03,0xb1f740b4
- );
+ );}
 }
 
 #hash fields :
@@ -97,9 +105,9 @@ INIT
 # serial	serial number (binary 4 bytes)
 # seg_table	segmentation table of last read page
 # granule	granule of last read page
-# info		-> hash containing : version channels rate bitrate_upper bitrate_nominal bitrate_lower seconds mmss
-# comments	-> hash of arrays
-# CommentsOrder -> list of keys
+# info		-> hash containing : version channels rate bitrate_upper bitrate_nominal bitrate_lower seconds
+# comments	-> hash of arrays (lowercase keys)
+# CommentsOrder -> list of keys (mixed-case keys)
 # commentpack_size
 # vorbis_string
 # stream_vers
@@ -110,7 +118,6 @@ sub new
 {   my ($class,$file)=@_;
     my $self=bless {}, $class;
 
-    local $_;
     # check that the file exists and is readable
     unless ( -e $file && -r $file )
     {	warn "File '$file' does not exist or cannot be read.\n";
@@ -133,11 +140,9 @@ sub new
 	#calulate length
 	last unless $self->{info}{rate};# && $self->{end};
 	my @granule=unpack 'C*',$self->{granule};
-	my $l=pop @granule;
+	my $l=0;
 	$l=$l*256+$_ for reverse @granule;
 	$self->{info}{seconds}=my$s=$l/$self->{info}{rate};
-	$self->{info}{mmss}=sprintf '%d:%02d',$s/60,$s%60;
-	#warn "length : ".$self->{info}{mmss}."\n";
     }
 
     $self->_close;
@@ -166,7 +171,7 @@ sub _openw
 	until (open $fh,$m,$file)
 	{	my $err="Error opening '$file' for writing :\n$!";
 		warn $err."\n";
-		return undef unless $self->{errorsub} && &{ $self->{errorsub} }($err) eq 'yes';
+		return undef unless $self->{errorsub} && $self->{errorsub}($err) eq 'yes';
 	}
 	binmode $fh;
 	unless ($tmp)
@@ -182,17 +187,15 @@ sub _close
 	close delete($self->{fileHandle});
 }
 
-sub write_file	# experimental
+sub write_file
 {	my $self=shift;
-	local $_;
 	my $newcom_packref=_PackComments($self);
 	#warn "old size $self->{commentpack_size}, need : ".length($$newcom_packref)."\n";
 	if ( $self->{commentpack_size} >= length $$newcom_packref)
 	{	warn "in place editing.\n";
 		my $left=length $$newcom_packref;
 		my $offset2=0;
-		my $fh=$self->_openw or return undef;
-		return undef unless defined $fh;
+		my $fh=$self->_openw or return;
 		_read_packet($self,PACKET_INFO);	#skip first page
 		while ($left)
 		{ my $pos=tell $fh;
@@ -208,9 +211,8 @@ sub write_file	# experimental
 		$self->_close;
 		return;
 	}
-	my $INfh=$self->_open or return undef;
-	my $OUTfh=$self->_openw(1) or return undef;	#open .TEMP file
-	return undef unless defined $INfh && defined $OUTfh;
+	my $INfh=$self->_open or return;
+	my $OUTfh=$self->_openw(1) or return;	#open .TEMP file
 
 	my $version=chr $self->{stream_vers};
 	my $serial=$self->{serial};
@@ -281,6 +283,8 @@ sub write_file	# experimental
 	close $OUTfh;
 	warn "replacing old file with new file.\n";
 	unlink $self->{filename} && rename $self->{filename}.'.TEMP',$self->{filename};
+	%$self=(); #destroy the object to make sure it is not reused as many of its data are now invalid
+	return 1;
 }
 
 sub _ReadPage
@@ -313,11 +317,8 @@ sub _ReadInfo
 	# 8) [blocksize_1] = 2 exponent (read 4 bits as unsigned integer)
 	# 9) [framing_flag] = read one bit
 	if ( my $packref=_read_packet($self,PACKET_INFO) )
-	{	@_=unpack 'x7 VCV V3 C',$$packref;
-		my %info;
-		#print "info : ".join(' ',@_)."\n";
-		$info{$_}=shift @_
-			for (qw/version channels rate bitrate_upper bitrate_nominal bitrate_lower/);
+	{	my %info;
+		@info{qw/version channels rate bitrate_upper bitrate_nominal bitrate_lower/}= unpack 'x7 VCV V3 C',$$packref;
 		return \%info;
 	}
 	else
@@ -330,26 +331,37 @@ sub _ReadComments
 {	my $self=$_[0];
 	if ( my $packref= _read_packet($self,PACKET_COMMENT) )
 	{	$self->{commentpack_size}=length $$packref;
-		my ($vendor,@comlist)=eval { unpack 'x7 V/a V/(V/a)',$$packref; };
+		my ($vstring,@comlist)=eval { unpack 'x7 V/a V/(V/a)',$$packref; };
 		if ($@) { warn "Comments corrupted\n"; return undef; }
 		# Comments vendor strings I have found
 		# 'Xiph.Org libVorbis I 20030909' : 1.0.1
 		# 'Xiph.Org libVorbis I 20020717' : 1.0 release of libvorbis
 		# 'Xiphophorus libVorbis I 200xxxxx' : 1.0_beta1 to 1.0_rc3
 		# 'AO; aoTuV b3 [20041120] (based on Xiph.Org's libVorbis)'
-		$self->{vorbis_string}=$vendor;
-		unless ($vendor=~m/^Xiph.* libVorbis I (\d{8})/)
-		 { warn "unknown comments vendor string : $vendor\n"; }#return undef; }
-		#warn "comment version $1\n";
+		$self->{vorbis_string}=$vstring;
+		if ($::debug && $vstring!~m/^Xiph.* libVorbis I (\d{8})/)
+		 { warn "unknown comments vendor string : $vstring\n"; }
 		my %comments;
+		my @order;
+		$self->{CommentsOrder}=\@order;
 		for my $kv (@comlist)
 		{	unless ($kv=~m/^([^=]+)=(.*)$/s) { warn "comment invalid - skipped\n"; next; }
-			my $key=lc$1;
+			my $key=$1;
 			my $val=decode('utf-8', $2);
 			#warn "$key = $val\n";
-			push @{ $comments{$key} },$val;
-			my $nb=@{ $comments{$key} }-1;
-			push @{$self->{CommentsOrder}}, $key;
+			push @{ $comments{lc$key} },$val;
+			push @order, $key;
+		}
+		if (my $covers=$comments{coverart})	#upgrade old embedded pictures format to metadata_block_picture
+		{	@order= grep !m/^coverart/i, @order;
+			for my $i (0..$#$covers)
+			{	my $data= $comments{"coverart"}[$i];
+				next unless $data;
+				my @val= ( map( $comments{"coverart$_"}[$i], qw/mime type description/ ), decode_base64($data) );
+				push @{$comments{metadata_block_picture}}, \@val;
+				push @order, 'METADATA_BLOCK_PICTURE';
+			}
+			delete $comments{"coverart$_"} for qw/mime type description/,'';
 		}
 		return \%comments;
 	}
@@ -363,12 +375,18 @@ sub _PackComments
 	my @comments;
 	my %count;
 	for my $key ( @{$self->{CommentsOrder}} )
-	{	my $nb=$count{$key}++ || 0;
-		my $val=$self->{comments}{$key}[$nb];
+	{	my $nb=$count{lc$key}++ || 0;
+		my $val=$self->{comments}{lc$key}[$nb];
 		next unless defined $val;
-		push @comments,$key.'='.$val;
+		$key=encode('ascii',$key);
+		$key=~tr/\x20-\x7D/?/c; $key=~tr/=/?/; #replace characters that are not allowed by '?'
+		if (uc$key eq 'METADATA_BLOCK_PICTURE' && ref $val)
+		{	$val= Tag::Flac::_PackPicture($val);
+			$val= encode_base64($$val);
+		}
+		push @comments,$key.'='.encode('utf8',$val);
 	}
-	my $packet=pack 'Ca6 V/a* V (V/a*)*',PACKET_COMMENT,'vorbis',$self->{vorbis_string},scalar @comments,@comments;
+	my $packet=pack 'Ca6 V/a* V (V/a*)*',PACKET_COMMENT,'vorbis',$self->{vorbis_string},scalar @comments, @comments;
 	$packet.="\x01"; #framing_flag
 	return \$packet;
 }
@@ -377,29 +395,20 @@ sub edit
 {	my ($self,$key,$nb,$val)=@_;
 	$nb||=0;
 	my $aref=$self->{comments}{lc$key};
-	return undef unless $aref &&  @$aref >=$nb;	#FIXME more sanity checks
-	my $old=$aref->[$nb];
-	$aref->[$nb]=$val;
-	return $old;
+	return unless $aref &&  @$aref >=$nb;
+	$aref->[$nb]= $val;
+	return 1;
 }
 sub add
-{	my $self=shift;
-	if (@_%2) {warn 'invalid call to add comments, args must be pairs of $key,$val'."\n"; return undef;}
-	while (@_)
-	{	my $key=lc shift; my $val=shift;
-		push @{ $self->{comments}{$key} },$val;
-		push @{$self->{CommentsOrder}}, $key;
-	}
+{	my ($self,$key,$val)=@_;
+	push @{ $self->{comments}{lc$key} }, $val;
+	push @{$self->{CommentsOrder}}, $key;
 	return 1;
 }
 sub insert	#same as add but put it first (of its kind)
-{	my $self=shift;
-	if (@_%2) {warn 'invalid call to insert comments, args must be pairs of $key,$val'."\n"; return undef;}
-	while (@_)
-	{	my $key=lc shift; my $val=shift;
-		unshift @{ $self->{comments}{$key} },$val;
-		push @{$self->{CommentsOrder}}, $key;
-	}
+{	my ($self,$key,$val)=@_;
+	unshift @{ $self->{comments}{lc$key} }, $val;
+	push @{$self->{CommentsOrder}}, $key;
 	return 1;
 }
 
@@ -411,6 +420,23 @@ sub remove_all
 	return 1;
 }
 
+sub get_keys
+{	keys %{ $_[0]{comments} };
+}
+sub get_values
+{	my ($self,$key)=($_[0],lc$_[1]);
+	my $v= $self->{comments}{$key};
+	return () unless $v;
+	if ($key eq 'metadata_block_picture')
+	{	for my $val (@$v)
+		{	next if ref $val or !defined $val;
+			my $dec=decode_base64($val);
+			$val= $dec ? Tag::Flac::_ReadPicture(\$dec) : undef;
+		}
+	}
+	return grep defined, @$v;
+}
+
 sub remove
 {	my ($self,$key,$nb)=@_;
 	return undef unless defined $key and $nb=~m/^\d*$/;
@@ -419,42 +445,8 @@ sub remove
 	my $val=$self->{comments}{$key}[$nb];
 	unless (defined $val) {warn "comment to delete not found\n"; return undef; }
 	$self->{comments}{$key}[$nb]=undef;
-	#return 1;
-	return $val;
-}
-sub clear_comments	# Untested
-{	my $self=shift;
-	if (@_)
-	{	for my $key (@_)
-		{	$key=lc$key;
-			return undef unless exists $self->{comments}{$key};
-			delete $self->{comments}{$key};
-			@{$self->{CommentsOrder}}=grep $_ ne $key, @{$self->{CommentsOrder}};
-		}
-	}
-	else
-	{	$self->{comments}={};
-		$self->{CommentsOrder}=[];
-	}
 	return 1;
 }
-sub listkeys	#former comment_tags
-{	my $self=shift;
-	return keys %{ $self->{comments} };
-}
-sub keyvalues	#former comment
-{	my ($self,$key)=@_;
-	return undef unless exists $self->{comments}{$key};
-	return @{ $self->{comments}{$key} };
-}
-sub info
-{	my $self=shift;
-	return $self->{info} unless @_;
-	#return map $self->{info}{$_}, @_;
-	my $key=shift;
-	return $self->{info}{$key};
-}
-sub path { shift->{filename}; }
 
 sub _read_packet
 {	my $self=shift;
@@ -515,16 +507,19 @@ sub _read_page_header
 	return 1;
 }
 
-sub _recompute_page_crc		# __SLOW__ in perl
+sub _recompute_page_crc
 { my $pageref=$_[0];
 
   #warn 'old crc : ',unpack('V',substr($$pageref,22,4)),"\n";
   substr $$pageref,22,4,"\x00\x00\x00\x00";
   my $crc=0;
-  # $crc=($crc<<8)^vec($crc_lookup, ($crc>>24)^vec($$pageref,$_,8) ,32); # a bit slower
-  #$crc=($crc<<8)^$crc_lookup[ ($crc>>24)^vec($$pageref,$_,8) ] #doesn't work if perl use 64bits
-  $crc=(($crc<<8)&0xffffffff)^$crc_lookup[ ($crc>>24)^vec($$pageref,$_,8) ]
+  if ($digestcrc) { $digestcrc->add($$pageref); $crc=$digestcrc->digest; }
+  else			# pure-perl : SLOW
+  {	 #$crc=($crc<<8)^vec($crc_lookup, ($crc>>24)^vec($$pageref,$_,8) ,32); # a bit slower
+	 #$crc=($crc<<8)^$crc_lookup[ ($crc>>24)^vec($$pageref,$_,8) ] #doesn't work if perl use 64bits
+	 $crc=(($crc<<8)&0xffffffff)^$crc_lookup[ ($crc>>24)^vec($$pageref,$_,8) ]
   	for (0 .. length($$pageref)-1);
+  }
   #warn "new crc : $crc\n";
   substr $$pageref,22,4,pack('V',$crc);
 }
