@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2007 Quentin Sculo <squentin@free.fr>
+# Copyright (C) 2005-2009 Quentin Sculo <squentin@free.fr>
 #
 # This file is part of Gmusicbrowser.
 # Gmusicbrowser is free software; you can redistribute it and/or modify
@@ -6,22 +6,46 @@
 # published by the Free Software Foundation
 
 =gmbplugin LYRICS
-Lyrics
-Lyrics plugin
-Display lyrics of the current song using lyrc.com.ar
+name	Lyrics
+title	Lyrics plugin
+desc	Search and display lyrics
 =cut
 
 package GMB::Plugin::LYRICS;
 use strict;
 use warnings;
-require 'simple_http.pm';
+require $::HTTP_module;
+our @ISA;
+BEGIN {push @ISA,'GMB::Context';}
 use base 'Gtk2::VBox';
 use constant
 {	OPT	=> 'PLUGIN_LYRICS_', # MUST begin by PLUGIN_ followed by the plugin ID / package name
-	title	=> _"Lyrics",
 };
 
-::SetDefaultOptions(OPT, FontSize => 10, PathFile => "~/.lyrics/%a/%t.lyric", LyricSite => 'lyrc');
+my $notfound=_"Not found";
+
+my @justification=
+(	left	=> _"Left aligned",
+	center	=> _"Centered",
+	right	=> _"Right aligned",
+	fill	=> _"Justified",
+);
+
+my @ContextMenuAppend=
+(	{	label	=> _"Scroll with song",		check	=> sub { $_[0]{self}{AutoScroll} },
+		code	=> sub { $::Options{OPT.'AutoScroll'}= $_[1]; $_[0]{self}->SetAutoScroll; },
+	},
+	{	label	=> _"Hide toolbar",		check	=> sub { $_[0]{self}{HideToolbar} },
+		code	=> sub { $_[0]{self}->SetToolbarHide($_[1]);  },
+	},
+	{	label	=> _"Choose font...",
+		code	=> sub { $_[0]{self}->ChooseFont; },
+	},
+	{	label	=> _"Lyrics alignement",	check	=> sub { $_[0]{self}{justification} },
+		submenu	=> \@justification,	submenu_reverse => 1,	submenu_ordered_hash=>1,
+		code	=> sub { $_[0]{self}{textview}->set_justification( $_[0]{self}{justification}=$_[1] );  },
+	},
+);
 
 my %sites=	# id => [name,url,?post?,function]	if the function return 1 => lyrics can be saved
 (	#lyrc	=>	['lyrc','http://lyrc.com.ar/en/tema1en.php','artist=%a&songname=%s'],
@@ -33,78 +57,113 @@ my %sites=	# id => [name,url,?post?,function]	if the function return 1 => lyrics
 		}],
 	#leoslyrics =>	['leolyrics','http://api.leoslyrics.com/api_search.php?artist=%a&songtitle=%s'],
 	#google	=>	['google','http://www.google.com/search?q="%a"+"%s"'],
-	#googlemusic =>	['google music','http://www.google.com/musicsearch?q="%a"+"%s"'],
 	lyriki	=>	['lyriki','http://lyriki.com/index.php?title=%a:%s',undef,
 		sub { my $no= $_[0]=~m/<div class="noarticletext">/s; $_[0]=~s/^.*<!--\s*start content\s*-->(.*?)<!--\s*end content\s*-->.*$/$1/s && !$no; }],
 	lyricsplugin => [lyricsplugin => 'http://www.lyricsplugin.com/winamp03/plugin/?title=%s&artist=%a',undef,
 			sub { my $ok=$_[0]=~m#<div id="lyrics">.*\w\n.*\w.*</div>#s; $_[0]=~s/<div id="admin".*$//s if $ok; return $ok; }],
-	lyricssongs =>	['lyrics-songs','http://www.lyrics-songs.com/winamp.php?musica=%s&artista=%a',undef,
-			sub { $_[0]=~s#<img src='p_bg2.gif'[^/]*/>##si; return 0 }], #remove image, return always 0 as lyrics-songs sometimes guess when the song is not found
-	#lyricwiki =>	[lyricwiki => 'http://lyricwiki.org/%a:%s'], # s#<div id="lyric">(.+?)</div>#$1#
-	lyricwikiapi => [lyricwiki => 'http://lyricwiki.org/api.php?artist=%a&song=%s&fmt=html',undef,
-		sub { $_[0]!~m#<pre>\W*Not found\W*</pre>#s }],
+	lyricssongs =>	['lyrics-songs','http://letras.terra.com.br/winamp.php?musica=%s&artista=%a',undef,
+			sub { my $l=html_extract($_[0],div=>'letra'); my $ref=\$_[0]; $$ref=$l ? $l : $notfound; return !!$l }],
+	lyricwiki =>	[lyricwiki => 'http://lyrics.wikia.com/%a:%s',undef,
+			 sub {	return 0,'http://lyrics.wikia.com/'.$1 if $_[0]=~m#<span class="redirectText"><a href="/([^"]+)"#;
+				$_[0]=~s!.*<div class='lyricbox'>.*?((?:&\#\d+;|<br ?/>){5,}).*!$1!s; #keep only the "lyric box"
+				return 0 if $_[0]=~m/&#91;&#46;&#46;&#46;&#93;<br/; # truncated lyrics : "[...]" => not auto-saved
+				return !!$1;
+			}],
+	#lyricwikiapi => [lyricwiki => 'http://lyricwiki.org/api.php?artist=%a&song=%s&fmt=html',undef,
+	#	sub { $_[0]!~m#<pre>\W*Not found\W*</pre>#s }],
 	#azlyrics => [ azlyrics => 'http://search.azlyrics.com/cgi-bin/azseek.cgi?q="%a"+"%s"'],
+	#Lyricsfly ?
 );
 
+$::Options{OPT.'Font'} ||= delete $::Options{OPT.'FontSize'};	#for versions <1.1.6
 
+if (my $site=$::Options{OPT.'LyricSite'}) { delete $::Options{OPT.'LyricSite'} unless exists $sites{$site} } #reset selected site if no longer defined
+::SetDefaultOptions(OPT, font => 10, PathFile => "~/.lyrics/%a/%t.lyric", LyricSite => 'lyricsplugin');
+
+
+my $lyricswidget=
+{	class		=> __PACKAGE__,
+	tabicon		=> 'gmb-lyrics',		# no icon by that name by default
+	tabtitle	=> _"Lyrics",
+	saveoptions	=> 'HideToolbar font follow justification',
+	schange		=> \&SongChanged,
+	group		=> 'Play',
+	autoadd_type	=> 'context page lyrics text',
+	justification	=> 'left',
+};
 
 sub Start
-{	GMB::Context::AddPackage(__PACKAGE__,'Lyrics');
+{	Layout::RegisterWidget(PluginLyrics => $lyricswidget);
 }
 sub Stop
-{	GMB::Context::RemovePackage('Lyrics');
+{	Layout::RegisterWidget(PluginLyrics => undef);
 }
 
 sub new
-{	my ($class,$widget_options)=@_;
+{	my ($class,$options)=@_;
 	my $self = bless Gtk2::VBox->new(0,0), $class;
-	$self->{widget_options}=$widget_options;
-	my $toolbar=Gtk2::HBox->new;
+	$options->{follow}=1 if not exists $options->{follow};
+	$self->{$_}=$options->{$_} for qw/HideToolbar follow group font justification/;
+
 	my $textview=Gtk2::TextView->new;
+	$self->signal_connect(map => sub { $_[0]->SongChanged( ::GetSelID($_[0]) ); });
 	$textview->signal_connect(button_release_event	=> \&button_release_cb);
 	$textview->signal_connect(motion_notify_event 	=> \&update_cursor_cb);
 	$textview->signal_connect(visibility_notify_event=>\&update_cursor_cb);
 	$textview->signal_connect(scroll_event		=> \&scroll_cb);
 	$textview->signal_connect(populate_popup	=> \&populate_popup_cb);
 	$textview->set_wrap_mode('word');
+	$textview->set_justification( $self->{justification} );
 	$textview->set_left_margin(5);
+	$textview->set_right_margin(5);
+	if (my $color= $options->{color} || $options->{DefaultFontColor})
+	{	$textview->modify_text('normal', Gtk2::Gdk::Color->parse($color) );
+	}
 	$self->{buffer}=$textview->get_buffer;
 	$self->{textview}=$textview;
 	my $sw=Gtk2::ScrolledWindow->new;
-	$sw->set_shadow_type('etched-in');
+	$sw->set_shadow_type( $options->{shadow} || 'etched-in');
 	$sw->set_policy('automatic','automatic');
 	$sw->add($textview);
-	my $bsave=::NewIconButton('gtk-save',_"Save",\&Save_text,'none');
-	$::Tooltips->set_tip($bsave,_"Save lyrics");
-	$self->{buffer}->signal_connect(modified_changed => sub {$bsave->set_sensitive($_[0]->get_modified);} );
-	my $backb=::NewIconButton('gtk-go-back',undef,sub
-		{	my $self=::find_ancestor($_[0],__PACKAGE__);
-			my $url=pop @{$self->{history}};
-			$_[0]->set_sensitive(0) unless @{$self->{history}};
-			$self->{lastokurl}=undef;
-			$self->load_url($url) if $url;
-		},'none',_"Previous page");
-	my $breload=::NewIconButton('gtk-refresh',_"Refresh",sub
-		{	my $self=::find_ancestor($_[0],__PACKAGE__);
-			my $ID=delete $self->{ID};
-			$self->SongChanged($ID,1);
-		},'none');
-	my $source=::NewPrefCombo( OPT.'LyricSite', {map {$_=>$sites{$_}[0]} keys %sites} , undef, \&SetSource);
-	my $zoom=::NewPrefSpinButton( OPT.'FontSize', sub {SetFont($textview)},1,0,4,50,1,5);
-	$::Tooltips->set_tip($zoom, _"Font size");
-	my $scroll=::NewPrefCheckButton( OPT.'AutoScroll', _"Auto-scroll",\&SetAutoScroll,_"Scroll with the song");
-	$toolbar->pack_start($_,0,0,2) for $backb,$bsave,$breload,$zoom,$scroll,$source;
+	my $toolbar=Gtk2::Toolbar->new;
+	$toolbar->set_style( $options->{ToolbarStyle}||'both-horiz' );
+	$toolbar->set_icon_size( $options->{ToolbarSize}||'small-toolbar' );
+	for my $aref
+	(	[backb => 'gtk-go-back',\&Back_cb,	_"Previous page"],
+		[saveb => 'gtk-save',	\&Save_text,	_"Save",	_"Save lyrics"],
+		[undef, 'gtk-refresh',	\&Refresh_cb,	_"Refresh"],
+	)
+	{	my ($key,$stock,$cb,$label,$tip)=@$aref;
+		my $item=Gtk2::ToolButton->new_from_stock($stock);
+		$item->set_label($label);
+		$item->signal_connect(clicked => $cb);
+		$item->set_tooltip_text($tip) if $tip;
+		$toolbar->insert($item,-1);
+		::weaken( $self->{$key}=$item ) if $key;
+	}
+	$self->{saveb}->set_is_important(1);
+
+	# create follow toggle button, function from GMB::Context
+	my $follow=$self->new_follow_toolitem;
+
+	my $adj= $self->{fontsize_adj}= Gtk2::Adjustment->new(10,4,80,1,5,0);
+	my $zoom=Gtk2::ToolItem->new;
+	$zoom->add( Gtk2::SpinButton->new($adj,1,0) );
+	$zoom->set_tooltip_text(_"Font size");
+	my $source=::NewPrefCombo( OPT.'LyricSite', {map {$_=>$sites{$_}[0]} keys %sites} ,cb => \&Refresh_cb, toolitem=> _"Lyrics source");
+	my $scroll=::NewPrefCheckButton( OPT.'AutoScroll', _"Auto-scroll", cb=>\&SetAutoScroll, tip=>_"Scroll with the song", toolitem=>1);
+	$toolbar->insert($_,-1) for $follow,$zoom,$scroll,$source;
+
 	$self->pack_start($toolbar,0,0,0);
 	$self->add($sw);
-	$self->show_all;
-	$toolbar->set_no_show_all(1);
-	$toolbar->hide if $self->{widget_options}{HideToolbar};
 	$self->{toolbar}=$toolbar;
 	$self->signal_connect(destroy => \&destroy_event_cb);
 
-	$backb->set_sensitive(0);
-	$self->{backb}=$backb;
-	SetFont($textview);
+	$self->{buffer}->signal_connect(modified_changed => sub {$_[1]->set_sensitive($_[0]->get_modified);}, $self->{saveb});
+	$self->{backb}->set_sensitive(0);
+	$self->SetFont;
+	$adj->signal_connect(value_changed=> sub { $self->SetFont($_[0]->get_value) });
+	$self->SetToolbarHide($self->{HideToolbar});
 	$self->SetAutoScroll;
 
 	return $self;
@@ -123,14 +182,30 @@ sub cancel
 
 sub prefbox
 {	my $vbox=Gtk2::VBox->new(::FALSE, 2);
-	my $entry=::NewPrefEntry(OPT.'PathFile' => _"Load/Save lyrics in :");
-	my $autosave=::NewPrefCheckButton(OPT.'AutoSave' => _"Auto-save positive finds",undef,_"only works with some lyrics source and when the lyrics tab is displayed");
+	my $entry=::NewPrefEntry(OPT.'PathFile' => _"Load/Save lyrics in :", width=>30);
+	my $preview= Label::Preview->new(preview => \&filename_preview, event => 'CurSong Option', noescape=>1,wrap=>1);
+	my $autosave=::NewPrefCheckButton(OPT.'AutoSave' => _"Auto-save positive finds", tip=>_"only works with some lyrics source and when the lyrics tab is displayed");
 	my $Bopen=Gtk2::Button->new(_"open context window");
 	$Bopen->signal_connect(clicked => sub { ::ContextWindow; });
-	$vbox->pack_start($_,::FALSE,::FALSE,1) for $entry,$autosave,$Bopen;
+	$vbox->pack_start($_,::FALSE,::FALSE,1) for $entry,$preview,$autosave,$Bopen;
 	return $vbox;
 }
 
+sub filename_preview
+{	return '' unless defined $::SongID;
+	my $t=::pathfilefromformat( $::SongID, $::Options{OPT.'PathFile'}, undef,1);
+	$t= $t ? ::PangoEsc(_("example : ").$t) : "<i>".::PangoEsc(_"invalid pattern")."</i>";
+	return '<small>'.$t.'</small>';
+}
+
+sub SetToolbarHide
+{	my ($self,$hide)=@_;
+	$self->{HideToolbar}=$hide;
+	my $toolbar=$self->{toolbar};
+	if ($self->{HideToolbar})	{$toolbar->hide}
+	else				{$toolbar->set_no_show_all(0); $toolbar->show_all}
+}
+
 sub SetAutoScroll
 {	my $self=::find_ancestor($_[0],__PACKAGE__);
 	if ($self->{AutoScroll}=$::Options{OPT.'AutoScroll'})
@@ -138,36 +213,75 @@ sub SetAutoScroll
 	else	{ ::UnWatch($self,'Time'); };
 }
 sub SetFont
-{	my $textview=$_[0];
-	$textview->modify_font(Gtk2::Pango::FontDescription->from_string($::Options{OPT.'FontSize'}));
+{	my ($self,$newfont)=@_;
+	return if $self->{busy};
+	$self->{busy}=1;
+	my $textview=$self->{textview};
+	my $font= $self->{font} || $::Options{OPT.'Font'};
+	my $size;
+	if ($newfont)
+	{	if ($newfont=~m/\D/ || !$font) { $font=$newfont }
+		else { $size=$newfont }
+	}
+	my $fontdesc=Gtk2::Pango::FontDescription->from_string($font);
+	$fontdesc->set_size( $size * Gtk2::Pango->scale ) if $size;
+
+	# update spin button
+	$size= $fontdesc->get_size / Gtk2::Pango->scale;
+	my $adj=$self->{fontsize_adj};
+	$adj->set_value( $size );
+
+	$::Options{OPT.'Font'}= $self->{font}= $fontdesc->to_string;
+	$textview->modify_font($fontdesc);
+	delete $self->{busy};
+}
+sub ChooseFont
+{	my $self=shift;
+	my $dialog=Gtk2::FontSelectionDialog->new(_"Choose font for lyrics");
+	$dialog->set_font_name( $self->{font} );
+	my $response= $dialog->run;
+	if ($response eq 'ok')
+	{	$self->SetFont( $dialog->get_font_name );
+	}
+	$dialog->destroy;
 }
 
-sub SetSource
+sub Back_cb
+{	my $self=::find_ancestor($_[0],__PACKAGE__);
+	my $url=pop @{$self->{history}};
+	$_[0]->set_sensitive(0) unless @{$self->{history}};
+	$self->{lastokurl}=undef;
+	$self->load_url($url) if $url;
+}
+sub Refresh_cb
 {	my $self=::find_ancestor($_[0],__PACKAGE__);
 	$self->SongChanged($self->{ID},1);
 }
 
 sub SongChanged
 {	my ($self,$ID,$force)=@_;
+	return unless $self->mapped;
 	return unless defined $ID;
-	return if defined $self->{ID} && $ID==$self->{ID} && !$force;
+	return if defined $self->{ID} && !$force && ( $ID==$self->{ID} || !$self->{follow} );
+	$self->cancel;	#cancel any lyrics operation in progress on an a previous song
 	$self->{ID}=$ID;
 	$self->{time}=undef;
 
 	if (!$force)
-	{	my $file=::pathfilefromformat( $self->{ID}, $::Options{OPT.'PathFile'} );
+	{	my $file=::pathfilefromformat( $self->{ID}, $::Options{OPT.'PathFile'}, undef,1 );
 		if ($file && -r $file)
 		{	::IdleDo('8_lyrics'.$self,1000,\&load_file,$self,$file);
 			return
 		}
 	}
 
-	my $ref=$::Songs[$ID];
-	my $title=$ref->[::SONG_TITLE];
-	my $artist=$ref->[::SONG_ARTIST];
-	$_=::url_escapeall($_) for $title,$artist;
+	my ($title,$artist)= map ::url_escapeall($_), Songs::Get($ID,qw/title artist/);
 	my (undef,$url,$post,$check)=@{$sites{$::Options{OPT.'LyricSite'}}};
-	for ($url,$post) { next unless defined $_; s/%a/$artist/; s/%s/$title/; }
+	for my $val ($url,$post)
+	{	next unless defined $val;
+		if (ref $val) { $val= $val->(Songs::Get($ID,qw/title artist/),$ID); }
+		else {	$val=~s/%a/$artist/; $val=~s/%s/$title/; }
+	}
 	#$self->load_url($url,$post);
 	::IdleDo('8_lyrics'.$self,1000,\&load_url,$self,$url,$post,$check);
 }
@@ -182,7 +296,7 @@ sub TimeChanged		#scroll the text
 	return if $adj->get_value > $adj->upper - $adj->page_size;
 	my $delta=$::PlayTime - ($self->{time} || 0);
 	return if $delta <1;
-	my $inc=$delta / $::Songs[$::SongID][::SONG_LENGTH];
+	my $inc=$delta / Songs::Get($::SongID,'length');
 	$self->{time}=$::PlayTime;
 	$adj->set_value($adj->get_value+$inc*$range);
 }
@@ -191,23 +305,6 @@ sub populate_popup_cb
 {	my ($textview,$menu)=@_;
 	my $self=::find_ancestor($textview,__PACKAGE__);
 
-	my $item0=Gtk2::CheckMenuItem->new(_"Scroll with song");
-	$item0->set_active(1) if $self->{AutoScroll};
-	$item0->signal_connect(toggled => sub
-		{	my $self=$_[1];
-			$::Options{OPT.'AutoScroll'}=$_[0]->get_active;
-			$self->SetAutoScroll;
-		},$self);
-
-	my $item1=Gtk2::CheckMenuItem->new(_"Hide toolbar");
-	$item1->set_active(1) if $self->{widget_options}{HideToolbar};
-	$item1->signal_connect(toggled => sub
-		{	my $self=$_[1];
-			if ($self->{widget_options}{HideToolbar}^=1)	{$self->{toolbar}->hide}
-			else						{$self->{toolbar}->show}
-		},$self);
-	$menu->append($_) for Gtk2::SeparatorMenuItem->new, $item0,$item1;
-
 	# add menu items for links
 	my ($x,$y)=$textview->window_to_buffer_coords('widget',$textview->get_pointer);
 	if (my $url=$self->url_at_coords($x,$y))
@@ -222,9 +319,33 @@ sub populate_popup_cb
 		$menu->prepend($_) for Gtk2::SeparatorMenuItem->new, $item3,$item2;
 	}
 
+	$menu->append(Gtk2::SeparatorMenuItem->new);
+	::BuildMenu( \@ContextMenuAppend, { self=>$self, }, $menu );
+
 	$menu->show_all;
 }
 
+sub html_extract
+{	my ($data,$tag,$id)=@_;
+	my $re=qr/<\Q$tag\E [^>]*id="(\w+)"[^>]*>|<(\/)?\Q$tag\E>/i;
+	my ($start,$depth);
+	while ($data=~m/$re/g)
+	{	if ($2) #closing
+		{	if ($depth)
+			{	$depth--;
+				return substr $data,$start,$+[0]-$start unless $depth;
+			}
+		}
+		elsif ($depth)
+		{	$depth++
+		}
+		elsif (defined $1 && $1 eq $id)
+		{	$start=$-[0];
+			$depth=1;
+		}
+	}
+}
+
 sub load_url
 {	my ($self,$url,$post,$check)=@_;
 	$self->{buffer}->set_text(_"Loading...");
@@ -241,7 +362,7 @@ sub loaded #_very_ crude html to gtktextview renderer
 {	my ($self,$data,$type,$url)=@_;
 	delete $self->{waiting};
 	my $buffer=$self->{buffer};
-	unless ($data) { $data=_("Loading failed.").' <a href="'.$self->{url}.'">'._("retry").'</a>'; $type="text/html"; }
+	unless ($data) { $data=_("Loading failed.").qq( <a href="$self->{url}">)._("retry").'</a>'; $type="text/html"; }
 	$self->{url}=$url if $url; #for redirections
 	$buffer->delete($buffer->get_bounds);
 	my $encoding;
@@ -249,7 +370,7 @@ sub loaded #_very_ crude html to gtktextview renderer
 	if ($type && $type!~m#^text/html#)
 	{	if	($type=~m#^text/#)	{$buffer->set_text($data);}
 		elsif	($type=~m#^image/#)
-		{	my $loader=::LoadPixData($data);
+		{	my $loader= GMB::Picture::LoadPixData($data);
 			if (my $p=$loader->get_pixbuf) {$buffer->insert_pixbuf(0,$p);}
 		}
 		return;
@@ -260,8 +381,10 @@ sub loaded #_very_ crude html to gtktextview renderer
 	$data=Encode::decode($encoding,$data) if $encoding;
 
 	my $oklyrics;
-	$oklyrics=&{$self->{check}}($data) if $self->{check};
-#print "$data\n";
+	if (my $check=$self->{check})
+	{	($oklyrics,my $redirect)= $check->($data);
+		if ($redirect) { $self->load_url($redirect,undef,$check); return; }
+	}
 	if ($self->{lastokurl})
 	{	my $history=$self->{history}||=[];
 		push @$history,$self->{lastokurl};
@@ -281,6 +404,7 @@ sub loaded #_very_ crude html to gtktextview renderer
 	while (defined($_=shift @l))
 	{	s/[\r\n]//g;
 		s/\s+/ /g;
+		next if $_ eq ' ';
 		s#&nbsp;# #gi;
 		s#<br(?: */)?>#\n#gi;
 		if ($_ eq '<pre>') {$_="\n".shift(@l)."\n"}
@@ -380,7 +504,7 @@ sub load_pixbuf
 	sub
 	{	$self->{waiting}=undef;
 		my $loader;
-		$loader=::LoadPixData($_[0]) if $_[0];
+		$loader= GMB::Picture::LoadPixData($_[0]) if $_[0];
 		if ($loader)
 		{	my $buffer=$self->{buffer};
 			my $iter=$buffer->get_iter_at_mark($mark);
@@ -429,11 +553,10 @@ sub load_pixbuf
 sub scroll_cb	#zoom with ctrl-wheel
 {	my ($textview,$event) = @_;
 	return 0 unless $event->state >= 'control-mask';
-	my $size=\$::Options{OPT.'FontSize'};
-	$$size+= $event->direction eq 'up' ? 1 : -1;
-	$$size=4 if $$size<4;
-	$$size=50 if $$size>50;
-	SetFont($textview);
+	my $self=::find_ancestor($textview,__PACKAGE__);
+	my $size= $self->{fontsize_adj}->get_value;
+	$size+= $event->direction eq 'up' ? 1 : -1;
+	$self->{fontsize_adj}->set_value($size);
 	return 1;
 }
 
@@ -454,7 +577,7 @@ sub url_at_coords
 	{	next unless $tag->{url};
 		if ($tag->{url}=~m/^#(\d+)?/) { $self->scrollto($1) if defined $1; last }
 		my $url= $self->full_url( $tag->{url} );
-		if ($url=~m#^http://www\.lyrc\.com\.ar/en/add/add\.php\?#) {::openurl( quotemeta($url) ); return}	#lyrc specific
+		if ($url=~m#^http://www\.lyrc\.com\.ar/en/add/add\.php\?#) {::openurl($url); return}	#lyrc specific
 		return $url;
 	}
 }
@@ -505,7 +628,7 @@ sub load_file
         $buffer->set_text($text);
 
 	#make the title and artist bigger and bold
-	my ($title,$artist)=@{ $::Songs[$self->{ID}] }[::SONG_TITLE,::SONG_ARTIST];
+	my ($title,$artist)= Songs::Get($self->{ID},qw/title artist/);
 	$title='' if $title!~m/\w\w/;
 	for ($title,$artist)
 	{	if (m/\w\w/) {s#\W+#\\W*#g}
@@ -526,7 +649,7 @@ sub Save_text
 	my $buffer=$self->{buffer};
 	my $text= $buffer->get_text($buffer->get_bounds, ::FALSE);
 	my $format=$::Options{OPT.'PathFile'};
-	my ($path,$file)=::pathfilefromformat( $self->{ID}, $format );
+	my ($path,$file)=::pathfilefromformat( $self->{ID}, $format, undef,1 );
 	unless ($path && $file) {::ErrorMessage(_("Error: invalid filename pattern")." : $format",$win); return}
 	my $res=::CreateDir($path,$win);
 	return unless $res eq 'ok';
@@ -534,6 +657,7 @@ sub Save_text
 	{	print $fh $text;
 		close $fh;
 		$buffer->set_modified(0);
+		warn "Saved lyrics in ".$path.::SLASH.$file."\n" if $::debug;
 	}
 	else {::ErrorMessage(::__x(_("Error saving lyrics in '{file}' :\n{error}"), file => $file, error => $!),$win);}
 }
