@@ -18,6 +18,7 @@ while (my $file=shift @files)
  warn "reading $file\n" if $verbose;
  open my$fh,$file  or die $!;
  $file=~s#^$path/##;
+ my ($layout_id,$layout_loc);
  while (<$fh>)
  {	if (!$version && m/VERSION *=> '([0-9]\.[0-9]+)'/) {$version=$1}
 
@@ -33,15 +34,20 @@ while (my $file=shift @files)
 	if (m/^=gmbplugin \D\w+/)
 	{	while (<$fh>)
 		{	last if m/^=cut/;
-			chomp;
-			$msgid{$_}.=" $file:$.";
+			$msgid{$1}.= " $file:$." if m/^\s*(?:name|title|desc)\s+(.+)/;
 		}
 	}
-	if ($file eq 'layouts')
-	{	if (m/^\[([^]]+)\]/)		{ $msgid{$1}.=" $file:$."; }
+	if ($file eq 'layouts') #id of layouts is used as default layout name => may need translation
+	{	if (m/^\[([^]]+)\]/)
+		{	$msgid{ $layout_id }.= $layout_loc if defined $layout_id;
+			$layout_id=$1;
+			$layout_loc=" $file:$.";
+		}
+		elsif (m/^\s*Name\s*=/) { $layout_id=undef } #layout has a name => no need to translate the id
 	}
  }
  close $fh;
+ $msgid{ $layout_id }.= $layout_loc if defined $layout_id; #for last layout
 }
 
 $version||="VERSION";
@@ -56,11 +62,11 @@ msgstr ""
 "POT-Creation-Date: '.$date.'\n"
 "PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\n"
 "Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
-"Language-Team: LANGUAGE <LL@li.org>\n"
+"Language-Team: LANGUAGE\n"
 "MIME-Version: 1.0\n"
-"Content-Type: text/plain; charset=CHARSET\n"
+"Content-Type: text/plain; charset=UTF-8\n"
 "Content-Transfer-Encoding: 8bit\n"
-"Plural-Forms: nplurals=2; plural=(n > 1);\n"
+"Plural-Forms: nplurals=2; plural=n != 1;\n"
 ';
 
 for my $msg (sort keys %msgid)
